# -*- python -*-
# $RCSfile: genericselectGUI.py,v $
# $Revision: 1.20 $
# $Author: langer $
# $Date: 2005/05/13 18:43:50 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


# Base class for Selection toolboxes.

from oof2.SWIG.common import switchboard
from oof2.SWIG.common.IO.GUI import rubberband
from oof2.common import debug
from oof2.common import guitop
from oof2.common import primitives
from oof2.common import utils
from oof2.common.IO.GUI import historian
from oof2.common.IO.GUI import mousehandler
from oof2.common.IO.GUI import regclassfactory
from oof2.common.IO.GUI import toolboxGUI
from gtk import *

class HistoricalSelection:
    def __init__(self, selectionMethod, points):
        # Store a copy of the selection method.
        self.selectionMethod = selectionMethod
        self.points = points            # mouse coordinates
    def __repr__(self):
        return "HistoricalSelection(%s, %s)" %\
               (self.selectionMethod, self.points)


# Base class common to all selection toolboxes.  Needs access to the
# selectionmethod registry in order to build the
# registeredclassfactory.

# Subclasses *must* provide:
#  getSource()  returns the Who object within which the selection is made
#  finish_up()  completes the selection operation
#  undoCB()     gtk callback for undo button
#  redoCB()
#  clearCB()
#  invertCB()

class GenericSelectToolboxGUI(toolboxGUI.GfxToolbox,
                              mousehandler.MouseHandler):
    def __init__(self, name, toolbox, method):
        debug.mainthreadTest()
        toolboxGUI.GfxToolbox.__init__(self, name, toolbox)
        self.method = method            # RegisteredClass of selection methods
        self.points = []                # locations of mouse events
        # Was a modifier key pressed during the last button event?
        self.shift = 0                 
        self.ctrl = 0

        outerbox = GtkVBox(spacing=2)
        self.gtk.add(outerbox)

        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        outerbox.add(scroll)
        self.selectionMethodFactory = regclassfactory.RegisteredClassFactory(
            method.registry, title="Method:")
        scroll.add_with_viewport(self.selectionMethodFactory.gtk)
        self.historian = historian.Historian(self.setHistory,
                                             self.sensitizeHistory)
        self.selectionMethodFactory.set_callback(self.historian.stateChangeCB)

        # Undo, Redo, Clear, and Invert buttons.  The callbacks for
        # these have to be defined in the derived classes.
        hbox = GtkHBox()
        outerbox.add(hbox, expand=0, fill=0)
        self.undobutton = GtkButton('Undo')
        self.redobutton = GtkButton('Redo')
        hbox.add(self.undobutton, expand=1, fill=0)
        hbox.add(self.redobutton, expand=1, fill=0)
        self.undobutton.connect('clicked', self.undoCB)
        self.redobutton.connect('clicked', self.redoCB)
        guitop.top().tooltip(self.undobutton,
                             "Undo the previous selection operation.")
        guitop.top().tooltip(self.redobutton,
                             "Redo an undone selection operation.")
        self.clearbutton = GtkButton('Clear')
        hbox.add(self.clearbutton, expand=1, fill=0)

        self.clearbutton.connect('clicked', self.clearCB)
        guitop.top().tooltip(self.clearbutton, "Unselect all objects.")

        self.invertbutton = GtkButton('Invert')
        hbox.add(self.invertbutton, expand=1, fill=0)
        self.invertbutton.connect('clicked', self.invertCB)
        guitop.top().tooltip(
            self.invertbutton,
            "Select all unselected objects, and deselect all selected objects.")

        # Selection history
        frame = GtkFrame('History')
        outerbox.add(frame, expand=0, fill=0)
        vbox = GtkVBox()

        frame.add(vbox)
        
        table = GtkTable(rows=2, cols=3)
        vbox.add(table)
        table.attach(GtkLabel('down'), 0,1, 0,1, xoptions=0, yoptions=0)
        table.attach(GtkLabel('up'), 0,1, 1,2, xoptions=0, yoptions=0)
        self.xdownentry = GtkEntry()
        self.ydownentry = GtkEntry()
        self.xupentry = GtkEntry()
        self.yupentry = GtkEntry()
        self.xdownentry.connect("changed", self.poschanged)
        self.ydownentry.connect("changed", self.poschanged)
        self.xupentry.connect("changed", self.poschanged)
        self.yupentry.connect("changed", self.poschanged)
        self.xdownentry.set_usize(25, -1)
        self.ydownentry.set_usize(25, -1)        
        self.xupentry.set_usize(25, -1)
        self.yupentry.set_usize(25, -1)        
        table.attach(self.xdownentry, 1,2, 0,1)
        table.attach(self.ydownentry, 2,3, 0,1)
        table.attach(self.xupentry, 1,2, 1,2)
        table.attach(self.yupentry, 2,3, 1,2)
        hbox = GtkHBox()
        vbox.add(hbox)
        self.prevmethodbutton = GtkButton()
        lbox = GtkHBox()
        lbox.add(GtkArrow(at=ARROW_LEFT))
        lbox.add(GtkLabel(' Prev'))
        self.prevmethodbutton.add(lbox)
        self.repeatbutton = GtkButton('Repeat')
        self.nextmethodbutton = GtkButton(label=None)
        lbox = GtkHBox()
        lbox.add(GtkLabel('Next '), expand=0, fill=0)
        lbox.add(GtkArrow(at=ARROW_RIGHT), expand=0,fill=0)
        self.nextmethodbutton.add(lbox)
        hbox.add(self.prevmethodbutton, expand=0, fill=0)
        hbox.add(self.repeatbutton, expand=1, fill=0)
        hbox.add(self.nextmethodbutton, expand=0, fill=0)
        self.repeatbutton.connect('clicked', self.repeatCB)
        self.repeatbutton.connect('button-release-event', self.repeateventCB)
        self.prevmethodbutton.connect('clicked', self.historian.prevCB)
        self.nextmethodbutton.connect('clicked', self.historian.nextCB)
        guitop.top().tooltip(self.prevmethodbutton,
                    "Recall the settings and mouse coordinates for the previous selection method.")
        guitop.top().tooltip(self.nextmethodbutton,
                    "Recall the settings and mouse coordinates for the next selection method.")
        guitop.top().tooltip(self.repeatbutton,
                             "Execute the selection method as if the mouse had been clicked at the above coordinates.  Hold the shift key to retain the previous selection.  Hold the control key to toggle the selection state of the selected pixels.")
        

        # Selection information
        hbox = GtkHBox()
        outerbox.add(hbox, expand=0, fill=0)
        hbox.add(GtkLabel('Selection size: '), expand=0, fill=0)
        self.sizetext = GtkEntry();
        hbox.add(self.sizetext, expand=1, fill=1)
        self.sizetext.set_editable(0)
        self.sizetext.set_usize(50, -1)
        self.setInfo()

        # switchboard callbacks
        self.sbcallbacks = [
            switchboard.requestCallback(method, self.updateSelectionMethods),
            switchboard.requestCallback((self.toolbox.gfxwindow(),
                                         'layers changed'), self.setInfo)
        ]

    def activate(self):
        toolboxGUI.GfxToolbox.activate(self)
        self.sensitize()
        self.sensitizeHistory()
        self.setInfo()

        self.gfxwindow().setMouseHandler(self)

    def deactivate(self):
        self.gfxwindow().setRubberband(rubberband.NoRubberBand())
        toolboxGUI.GfxToolbox.deactivate(self)

    def close(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        toolboxGUI.GfxToolbox.close(self)

    def getSourceName(self):
        source = self.getSource()
        if source is not None:
            return source.path()

    def layerChangeCB(self):
        # Called when layers have been added, removed, or moved in the gfxwindow
        self.sensitize()

    def newSelection(self, selectionMethod, pointlist): # switchboard callback
        if selectionMethod is not None:
            self.historian.record(HistoricalSelection(selectionMethod,
                                                      pointlist))
            self.setCoordDisplay(selectionMethod.getRegistration(), pointlist)
            self.selectionMethodFactory.setByRegistration(
                selectionMethod.getRegistration())

    def changedSelection(self):         # switchboard callback
        self.sensitize()
        self.setInfo()

    def poschanged(self, *args):
        self.sensitizeHistory()
            
    def setHistory(self, historicalSelection):
        self.selectionMethodFactory.set(historicalSelection.selectionMethod,
                                        interactive=1)
        self.setCoordDisplay(
            historicalSelection.selectionMethod.getRegistration(),
            historicalSelection.points)

    def sensitize(self):
        debug.mainthreadTest()
        source = self.getSource()
        ok = source is not None
        if source is not None:
            selection = source.getSelectionContext(**self.toolbox.extrakwargs)
        else:
            selection = None
        if source is None or selection is None:
            self.undobutton.set_sensitive(0)
            self.redobutton.set_sensitive(0)
            self.clearbutton.set_sensitive(0)
            self.invertbutton.set_sensitive(0)
        else:                           
            selection.begin_reading()
            try:
                self.undobutton.set_sensitive(selection.undoable())
                self.redobutton.set_sensitive(selection.redoable())
                self.clearbutton.set_sensitive(selection.clearable())
                self.invertbutton.set_sensitive(1)
            finally:
                selection.end_reading()
        self.sensitizeHistory()

    def sensitizeHistory(self):
        debug.mainthreadTest()
        self.nextmethodbutton.set_sensitive(self.historian.nextSensitive())
        self.prevmethodbutton.set_sensitive(self.historian.prevSensitive())
        self.repeatbutton.set_sensitive(len(self.historian) > 0
                                        and self.repeatable())

    def setInfo(self):
        debug.mainthreadTest()
        source = self.getSource()
        if source is not None:
            source.begin_reading()
            try:
                selection = source.getSelectionContext(
                    **self.toolbox.extrakwargs)
                ## Selection can be None because an Image must be created
                ## before a Microstructure, but the selection source for
                ## an Image is its Microstructure.
                if selection is not None:
                    selection.begin_reading()
                    try:
                        self.sizetext.set_text(`selection.size()`)
                    finally:
                        selection.end_reading()
                else:
                    self.sizetext.set_text('0')
            finally:
                source.end_reading()
        else:
            self.sizetext.set_text(self.toolbox.emptyMessage())

    def updateSelectionMethods(self):
        self.selectionMethodFactory.update(self.method.registry)

    ###################################

    # Command history
    
    def repeatCB(self, button):
        selmeth = self.selectionMethodFactory.getRegistration()
        if selmeth is not None:
            # Get the coordinates of the points from the widgets.  If
            # the selection method uses 'move' events, then those
            # points have to come from the Historian, because there's
            # no widget for them.
            if 'move' in selmeth.events:
                points = self.historian.current().points[:]
            else:
                points = [None]         # dummy array, will be filled later
            # Also, the list of points from the Historian might not
            # have the right length. 
            if 'up' in selmeth.events and 'down' in selmeth.events:
                # need two points, at least
                if len(points) == 1:
                    points.append(None)
            # Get the (possibly) edited values from the widgets
            try:
                if 'down' in selmeth.events:
                    points[0] = primitives.Point(
                        utils.OOFeval(self.xdownentry.get_text()),
                        utils.OOFeval(self.ydownentry.get_text()))
                if 'up' in selmeth.events:
                    points[-1] = primitives.Point(
                        utils.OOFeval(self.xupentry.get_text()),
                        utils.OOFeval(self.yupentry.get_text()))
            except:        # Shouldn't happen, if sensitization is working
                raise "Can't evaluate coordinates!"
            actual_who = self.getSource()
            if actual_who:
                self.selectionMethodFactory.set_defaults()
                menuitem = getattr(self.toolbox.menu, selmeth.name())
                self.toolbox.setSourceParams(menuitem, actual_who)
                menuitem.callWithDefaults(points=points,
                                          shift=self.shift, ctrl=self.ctrl)

    def repeateventCB(self, button, gdkevent):
        # Callback for 'button-release-event' on the 'Repeat' button.
        # This function is called before repeatCB and stores the
        # modifier key state.  This function is called whenever the
        # mouse is released as long as it was pushed on the 'Repeat'
        # button, so it can't be used as the actual button callback.
        # repeatCB is called only if the mouse is actually released on
        # the button, but doesn't have access to the modifier keys
        # like repeateventCB does.
        self.shift = (gdkevent.state & GDK.SHIFT_MASK != 0)
        self.ctrl = (gdkevent.state & GDK.CONTROL_MASK != 0)

    def repeatable(self):
        # Check that the mouse coord entry widgets contain appropriate
        # data.
        selmeth = self.selectionMethodFactory.getRegistration()
        try:
            if 'down' in selmeth.events:
                # OOFeval raises exceptions if the text is not a valid
                # Python expression
                utils.OOFeval(self.xdownentry.get_text())
                utils.OOFeval(self.ydownentry.get_text())
            if 'up' in selmeth.events:
                utils.OOFeval(self.xupentry.get_text())
                utils.OOFeval(self.yupentry.get_text())
        except:
            return 0
        return 1

    def setCoordDisplay(self, selectionMethodReg, points):
        debug.mainthreadTest()
##        self.xdownentry.set_text("%8g" % points[0].x)
##        self.ydownentry.set_text("%8g" % points[0].y)
##        self.xupentry.set_text("%8g" % points[-1].x)
##        self.yupentry.set_text("%8g" % points[-1].y)
        if 'down' in selectionMethodReg.events:
            self.xdownentry.set_text("%-8g" % points[0].x)
            self.ydownentry.set_text("%-8g" % points[0].y)
        else:
            self.xdownentry.set_text('--')
            self.ydownentry.set_text('--')
        if 'up' in selectionMethodReg.events:
            self.xupentry.set_text("%-8g" % points[-1].x)
            self.yupentry.set_text("%-8g" % points[-1].y)
        else:
            self.xupentry.set_text('--')
            self.yupentry.set_text('--')


    ###################################
        
    # MouseHandler functions
    
    def down(self, x, y, shift, ctrl):  # mouse down
        self.selmeth = self.selectionMethodFactory.getRegistration()
        self.selectionMethodFactory.set_defaults()
        self.gfxwindow().setRubberband(self.selmeth.getRubberBand(self.selmeth))
        # Start collecting points
        self.points = [primitives.Point(x,y)]

    def move(self, x, y, shift, ctrl):  # mouse move
        # Continue the collection of points, if it's been started...
        if self.points:
            self.points.append(primitives.Point(x,y))

    def up(self, x, y, shift, ctrl):    # mouse up
        # Finish the collection of points
        self.points.append(primitives.Point(x,y))
        if self.selmeth is not None:
            # Construct the list of points that the method needs
            ptlist = []
            if 'down' in self.selmeth.events:
                ptlist.append(self.points[0])
            if 'move' in self.selmeth.events and len(self.points) > 2:
                ptlist += self.points[1:-2]
            if 'up' in self.selmeth.events:
                ptlist.append(self.points[-1])

            source = self.getSource()

            if source:
                # We've done as much generic work as we can do --
                # we have a mouse-up event, a set of points,
                # a selection method, and a who.
                # Child classes know what to do next.
                self.finish_up(ptlist, shift, ctrl, self.selmeth)
                
            self.points = []            # get ready for next event
            
    def acceptEvent(self, eventtype):
        return 1
