# -*- python -*-
# $RCSfile: oof_mainiteration.py,v $
# $Revision: 1.20 $
# $Author: langer $
# $Date: 2005/05/13 18:43:51 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common.IO import threadstate
from oof2.common import debug
from oof2.common import mainthread
from oof2.common import thread_enable
import threading
import gtk

_inside_idle_callback = 0


# Thread-safe GTK mainiteration call -- wraps the gtk.mainiteration
# in threads_enter and threads_leave in the appropriate circumstances,
# i.e. when the main thread is in an idle-callback, or when threads
# are enabled and the caller is not the main thread.  This is
# required because both subthreads and idle_callbacks need to acquire
# the GTK lock before making GTK calls.
def mainiteration(block=gtk.TRUE):
     global _inside_idle_callback
     main = mainthread.mainthread()
     needgtklock = (thread_enable.query() and not main) or \
                   (_inside_idle_callback and main)
     if needgtklock:
##          debug.fmsg("mainiteration gtk.threads_enter()")
          gtk.threads_enter()

     gtk.mainiteration(block)

     if needgtklock:
          gtk.threads_leave()
##          debug.fmsg("mainiteration gtk.threads_leave()")

# Looping version, processes the full event queue.  This also
# wraps the events_pending call, which is necessary to prevent
# blocking.
def mainiteration_loop(block=gtk.TRUE):
     global _inside_idle_callback
     main = mainthread.mainthread()
     needgtklock = (thread_enable.query() and not main) or \
                   (_inside_idle_callback and main)
     if needgtklock:
##          debug.fmsg("mainiteration_loop gtk.threads_enter()")
          gtk.threads_enter()
     while gtk.events_pending():
          gtk.mainiteration(block)
     if needgtklock:
          gtk.threads_leave()
##          debug.fmsg("mainiteration_loop gtk.threads_leave()")

# The actual wrapped idle callback.  This needs to be a persistent
# object, because it may need to be called again, if it returns true.
# Mainly, this object exists to set the _inside_idle_callback flag.
class OOFIdleCallback:
     def __init__(self, func, args=(), kwargs={}):
          self.func = func
          self.args = args
          self.kwargs = kwargs
     def __call__(self):
          global _inside_idle_callback
          _inside_idle_callback = 1
          try:
               self.func(*self.args, **self.kwargs)
          finally:
               _inside_idle_callback = 0
          return gtk.FALSE              # don't repeat

# Adds a wrapped idle-function callback.
def oof_idle_add(func, args=(), kwargs={}):
     gtk.idle_add(OOFIdleCallback(func, args, kwargs))

###########################
     
class OOFIdleBlockCallback(OOFIdleCallback):
     def __init__(self, func, args=(), kwargs={}):
          OOFIdleCallback.__init__(self, func, args, kwargs)
          self.event = threading.Event()
          self.result = None
          self.callingthread = threadstate.findThreadNumber()
     def __call__(self):
          global _inside_idle_callback
          _inside_idle_callback = 1
          try:
##               debug.fmsg('Running blocking function from thread',
##                          self.callingthread)
               self.result = self.func(*self.args, **self.kwargs)
##               debug.fmsg('done running blocking function')
          finally:
               _inside_idle_callback = 0
               self.event.set()
          return gtk.FALSE              # don't repeat

def oof_idle_block_add(func, args=(), kwargs={}):
     callbackobj = OOFIdleBlockCallback(func, args, kwargs)
     callbackobj.event.clear()
     gtk.idle_add(callbackobj)
##     debug.fmsg('waiting for', func)
     callbackobj.event.wait()
##     debug.fmsg('done')
     return callbackobj.result
          
