# -*- python -*-
# $RCSfile: mpitools_.py,v $
# $Revision: 1.8 $
# $Author: langer $
# $Date: 2005/05/13 18:43:45 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import atexit, sys
from oof2.common.IO import menuparser

## wrapper for _mpi_initialize.
## mpi_finalize is registered so that so that when python exits it gets called

def mpi_initialize(argv):
    _mpi_initialize(argv)
    mpi_state = mpi_initialized()
    ## print "mpi state = ", mpi_state
    if mpi_state :
        atexit.register(mpi_finalize)
        mpi_barrier() ## is this really necessary?
        return
    print "MPI did not initialize!! Quitting..."
    sys.exit()


class SendPipe:
    ## Wrapper class for point-to-point
    ## encoded send communication.
    ## Its matching class is
    ## ReceivePipe
    def __init__(self, destination):
        self.destination = destination
        
    def newDestination(self,new_dest):
        ## How about Los Cabos, San Lucas?
        self.destination = new_dest
        
    def sendBytes(bytes, destination = None):
        if destination is not None:
            send_string(bytes, destination)
        else:
            send_string(bytes,self.destination)
            
    def sendLine(line, destination = None):
        self.sendBytes(line+"\n", dest)
            

class ReceivePipe (menuparser.InputSource):
    ## Wrapper class for point-to-point
    ## encoded receive communication.
    ## Its matching class is
    ## SendPipe
    def __init__(self, origin):
        self.origin = origin
        self.buffer = 800
        self.data = ""
    def getLine(self, origin = None):
        if origin is not None:
            return recieve_string(origin)
        else:
            return receive_string(self.origin)
    def getBytes(self, n, origin = None):
        ## first, if we are listening from a
        ## different processor, set the right
        ## direction
        real_org = self.origin
        if origin is not None:
            real_org = origin
        ## Now, read from the mpi channel until
        ## a number n of bytes has been received
        while 1:
            self._read(realorg)
            res = self.data[:n]
            self.data = self.data[n:]
            return res
    def _read(self, from_here):
        self.data += receive_string(from_here)
        
class MPI_Source(menuparser.InputSource):
    ## Wraper for a broadcast receive.
    ## Collective communication
    ## Its matching class is MPI_BcastPort
    def __init__(self, origin=0):
        self.origin = origin
        self.position = 0
        self.size = 0
        self.string = ""
    def getLine(self):
        return receive_broadcast_string(self.origin) ## all processes listen to front-end

    def getBytes(self, n):
        if self.position>= self.size:
            self.string = self.getLine()
            self.position = 0
            self.size = len(self.string)
        end = self.position + n
        if end > self.size:
            end = self.size
        result = self.string[self.position:end]
        self.position = end
        return result

class MPI_BcastPort:
    ## Wrapper for a broadcast send.
    ## Collective communication
    ## Its matching class is MPI_Source
    def __init__(self, origin = 0):
        self.origin = origin
        self.string = ""
        self.size = 0
        self.position = 0
    def sendLine(self, message): ## send a full string
        broadcast_string(message, self.origin)
    ## sending a set of bytes is a two step process:
    ## 1. acquire the string
    ## 2. send bytes of size N of the string until the
    ##    maximum size of the string has been reached.
    ## e.g.,
    ##      port.acquireLine(msg)
    ##      while port.sendBytes(N) is None:
    ##              pass
    def acquireLine(self, message):
        self.string = message
        self.size = len(message)
        self.position = 0
    def sendBytes(self, n):
        end = self.position + n
        if end > self.size:
            end = self.size
        result = self.string[self.position:end]
        self.position = end
        self.sendLine(result)
        if end > self.size:
            return 1
        return None




####################################################################

import traceback

class PyMPIException(MPIException):
    def __init__(self, message, processor):
        stack = traceback.extract_stack()
        # The actual error is two up from us in the stack.
        (file, lineno, func, txt) = stack[-2] 
        MPIException.__init__(self, message, file, lineno, processor)
    def __repr__(self):
        return self.message() 

def _MPIException___repr__(self):
    return "processor: " + self.processor + "\n" + self.message()+"\n("+self.filename()+":"+"%d"%self.lineno() + ")"

MPIException.__repr__ = _MPIException___repr__
