# -*- python -*-
# $RCSfile: quit.py,v $
# $Revision: 1.27 $
# $Author: langer $
# $Date: 2005/06/07 21:17:03 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from oof2.common import subthread
from oof2.common import debug
from oof2.common import excepthook
from oof2.common import guitop
from oof2.common import utils
from oof2.common.IO import threadmanager
from oof2.common.IO import mainmenu
from oof2.common import parallel_enable
from oof2.common.IO import threadmanager
from oof2.SWIG.common import switchboard
from oof2.SWIG.common import lock
import os
import sys


def quit(*args):
    # Called in text mode only.  *args allows it to be used as a menu callback.
    debug.mainthreadTest()
    if mainmenu.OOF.logChanged() and not quiet():
        answer = utils.OOFeval('raw_input("*Save log file? [Yn]: ")')
        if answer in ('','Y','y','yes','Yes', 'YES'):
            mainmenu.OOF.File.Save.Python_Log()
            mainmenu.cleanlog()
    cleanup()
    reallyquit()

def reallyquit():
    excepthook.restore_excepthook()
    sys.exit()
    

quitlock = lock.Lock()

## This lock is to prevent other actions that are being executed on a
## separate thread, such as saving microstructures, to be left
## incomplete due to the quitting sequence. If such menuitem is being
## executed while quitting, quit will wait on THIS lock until it
## finishes.

def cleanup():
    quitlock.acquire()
    if parallel_enable.query() == 1:
        try:
            from oof2.SWIG.common import mpitools
            if mpitools.rank() == 0:
                mainmenu.OOF.LoadData.IPC.Quit() ## tells back-end to start quit process
        except ImportError:
            pass
    # Turn off logging, so that window closing, etc. won't be logged.
    mainmenu.OOF.haltLog()
    subthread.miniThreadManager.quit()
    threadmanager.threadManager.quit()
    if guitop.top():
        guitop.top().stop()
        guitop.top().destroy()
    
    # Let other components that want to do something shut down
    # cleanly.  This signal should be requested with
    # requestCallbackInThread to guarantee that all callbacks are
    # called before the main thread shuts down.
    switchboard.notify('quitting time')

_quiet = 0

def quiet():
    return _quiet

def set_quiet():
    global _quiet
    _quiet = 1

def disallow():
    quitlock.acquire()

def allow():
    quitlock.release()

