// -*- C++ -*-
// $RCSfile: propertyoutput.h,v $
// $Revision: 1.17 $
// $Author: langer $
// $Date: 2004/07/29 21:42:34 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef PROPERTYOUTPUT_H
#define PROPERTYOUTPUT_H

#include <Python.h>

class PropertyOutput;
class PropertyOutputInit;

#include "common/pythonexportable.h"
#include <string>
#include <vector>

class Element;
class FEMesh;
class MasterCoord;
class OutputVal;
class OutputValue;
class Property;


class PropertyOutputInit {
public:
  virtual ~PropertyOutputInit() {}
  virtual OutputVal *operator()(const PropertyOutput*, const FEMesh*,
				const Element*, const MasterCoord&) const = 0;
};

class ScalarPropertyOutputInit : public PropertyOutputInit {
public:
  OutputVal *operator()(const PropertyOutput*, const FEMesh*,
			const Element*, const MasterCoord&) const;
};

class TwoVectorPropertyOutputInit : public PropertyOutputInit {
public:
  OutputVal *operator()(const PropertyOutput*, const FEMesh*,
			const Element*, const MasterCoord&) const;
};

class ThreeVectorPropertyOutputInit : public PropertyOutputInit {
public:
  OutputVal *operator()(const PropertyOutput*, const FEMesh*,
			const Element*, const MasterCoord&) const;
};

class SymmMatrix3PropertyOutputInit : public PropertyOutputInit {
public:
  OutputVal *operator()(const PropertyOutput*, const FEMesh*,
			const Element*, const MasterCoord&) const;
};

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

class PropertyOutput {
private:
  const std::string name_;
  const std::vector<PyObject*> params_;
  const int index_;
public:
  PropertyOutput(const std::string &name, const std::vector<PyObject*> *params);
  virtual ~PropertyOutput();
  const std::string &name() const { return name_; }
  int index() const { return index_; }
  const std::vector<PyObject*> &parameters() const { return params_; }
  std::vector<OutputValue> *evaluate(FEMesh*, Element*,
				     const PropertyOutputInit*,
				     const std::vector<MasterCoord*>*);
};

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

// Each kind of PropertyOutput must have a PropertyOutputRegistration
// that indicates the name of the output (eg "Elastic Energy" or
// "Total Strain"), its type (eg double or SymmMatrix3, via the
// OUTTYPE template parameter) and an initializer, which computes the
// property-independent part of the output. 

class PropertyOutputRegistration {
private:
  const std::string name_;
  const int index_;
  const PropertyOutputInit *initializer_;
  static std::vector<PropertyOutputRegistration*> &allPropertyOutputRegs();
public:
  PropertyOutputRegistration(const std::string &name,
			     const PropertyOutputInit *init);
  const std::string &name() const { return name_; }
  int index() const { return index_; }
  PropertyOutput *instantiate(const std::vector<PyObject*> *params) const {
    return new PropertyOutput(name(), params);
  }
  const PropertyOutputInit *initializer() const { return initializer_; }

  friend PropertyOutputRegistration *getPropertyOutputReg(const std::string&);
  friend int nPropertyOutputRegistrations();
};


#endif // PROPERTYOUTPUT_H
