# -*- python -*-
# $RCSfile: solver_.py,v $
# $Revision: 1.32 $
# $Author: redwing $
# $Date: 2005/05/18 18:18:24 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.common import cregisteredclass
from oof2.common import debug
from oof2.common.IO import parameter
from oof2.common.IO import xmlmenudump
from oof2.SWIG.common import devel
from oof2.SWIG.engine.preconditioner import PreconditionerPtr



def _apply(self, meshcontext):
    from oof2.SWIG.engine import solverdriver
    equilibrate = solverdriver.SolverDriver(
        self, meshcontext, solverdriver.SolverIterationManager(1))
    equilibrate.precompute()
    equilibrate.iterate()
    equilibrate.short_postcompute()

LinearSolverPtr.apply = _apply



cregisteredclass.registerCClass(LinearSolverPtr)

LinearSolverPtr.tip = "Solution methods for linear matrix equations."
LinearSolverPtr.discussion = \
              xmlmenudump.loadFile('DISCUSSIONS/engine/reg/linear_solver.xml')


gmresSolver = cregisteredclass.Registration(
    "GMRES",
    LinearSolverPtr,
    GMRESSolver,
    ordering=100,
    params=[
    parameter.IntParameter("max_iterations", 1000,
                           tip="maximum number of iterations to perform"),
    parameter.IntParameter("krylov_dimension", 100,
                           tip="Making the Krylov dimension bigger will improve convergence but use more memory."),
    parameter.FloatParameter("tolerance", 1.e-13,
                             tip="largest acceptable relative error"),
    parameter.RegisteredParameter("preconditioner",
                                  PreconditionerPtr,
                                  tip="Black magic for making the matrix more easily solvable")
    ],
    tip="Generalized Minimal Residual method.  An iterative method for non-symmetric matrices.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/gmres.xml')
    )

cregisteredclass.Registration(
    "BiCG",
    LinearSolverPtr,
    BiCGSolver,
    ordering=200,
    params=[
    parameter.IntParameter("max_iterations", 1000,
                           tip="maximum number of iterations to perform"),
    parameter.FloatParameter("tolerance", 1.e-13,
                             tip="largest acceptable relative error"),
    parameter.RegisteredParameter("preconditioner",
                                  PreconditionerPtr,
                                  tip="Black magic for making the matrix more easily solvable")
    ],
    tip="Bi-Conjugate Gradient method.  An iterative method for non-symmetric matrices.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/bicg.xml')
    )

cregisteredclass.Registration(
    "CG",
    LinearSolverPtr,
    CGSolver,
    ordering=300,
    params=[
    parameter.IntParameter("max_iterations",  1000,
                           tip="maximum number of iterations to perform"),
    parameter.FloatParameter("tolerance", 1.e-13,
                             tip="largest acceptable relative error"),
    parameter.RegisteredParameter("preconditioner",
                                  PreconditionerPtr,
                                  tip="Black magic for making the matrix more easily solvable")
    ],
    tip="Conjugate Gradient method for solving symmetric matrices.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/cg.xml')
    )

cregisteredclass.Registration(
    "BiCGStab",
    LinearSolverPtr,
    BiCGStabSolver,
    ordering=400,
    params=[
    parameter.IntParameter("max_iterations",  1000,
                           tip="maximum number of iterations to perform"),
    parameter.FloatParameter("tolerance", 1.e-13,
                             tip="largest acceptable relative error"),
    parameter.RegisteredParameter("preconditioner",
                                  PreconditionerPtr,
                                  tip="Black magic for making the matrix more easily solvable")
    ],
    tip="Stabilized Bi-Conjugate Gradient method for non-symmetric matrices.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/bicgstab.xml')
    )

cregisteredclass.Registration(
    "Direct",
    LinearSolverPtr,
    DirectSolver,
    ordering=1000,
    params=[],
    tip="A non-iterative non-sparse matrix solver using LU decomposition.  Uses a lot of memory. Not recommended if the finite element mesh is large.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/direct.xml')
    )






