# -*- python -*-
# $RCSfile: timedrivers.py,v $
# $Revision: 1.16 $
# $Author: reida $
# $Date: 2005/06/17 20:22:08 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from oof2.common import registeredclass
from oof2.common.IO import parameter
from oof2.SWIG.common import devel
from oof2.SWIG.common.IO import progressbar
from oof2.SWIG.engine import steppingscheme
from oof2.SWIG.common import switchboard
from oof2.SWIG.engine import timestepadapter
        

class TimeDriver (registeredclass.RegisteredClass):
    registry = []
    def redraw(self):
        switchboard.notify('redraw')

        
class Null(TimeDriver):
    tip = "Applies a linear or non-linear solver only once"
    def __init__(self):
        self.steppermethod = steppingscheme.NullScheme()
    def clean_up(self,  solverdriver, meshcontext, report_success = 1):
        solverdriver.update_solver_state(meshcontext, report_success)
        solverdriver.clear_memory() ## deletes temporary stiffness matrix and associated objects
        
    def apply(self, solverdriver, meshcontext, verbosity = 1,
              report_success = 1):
        solverdriver.apply(self.steppermethod, meshcontext,
                           verbosity = verbosity)
        self.clean_up(solverdriver, meshcontext, report_success)
        self.redraw()
        ## solverdriver.update_solver_state(meshcontext, report_success)
        ## solverdriver.clear_memory() ## deletes temporary stiffness matrix and associated objects
        ## self.redraw()
    def apply_one_step(self, solverdriver, meshcontext, verbosity = 1,
                       report_success = 1):
        solverdriver.apply(self.steppermethod, meshcontext,
                           verbosity = verbosity)


registeredclass.Registration(
    'Null',
    TimeDriver, ## superclass
    Null, ## subclass
    ordering = 100,
    params = [],
    tip="Single application of matrix driver"
    )

## OLD WAY OF DOING TIME STEPPING STARTS HERE ##
##    def apply(self, solverdriver, meshcontext, verbosity = 1, report_success = 1):
##        while self.steppermethod.iterate(meshcontext) is not None: 
##            ## iterate while the looping condition is not met
##            ## first solve the problem as if it were time independent
##            solverdriver.apply(self.steppermethod, meshcontext, verbosity, report_success)
##            self.steppermethod.update(meshcontext) ## solution is updated here
##        solverdriver.update_solver_state(meshcontext, report_success)
##        solverdriver.clear_memory() ## deletes temporary stiffness matrix and associated objects
##        solverdriver.redraw()
## OLD WAY OF DOING TIME STEPPING ENDS HERE ##

class Evolver(TimeDriver):
    tip = "For solving partial differential equations with implicit or explicit time dependence"
    def __init__(self, steppermethod, timeadaptermethod):
        self.steppermethod = steppermethod
        self.timeadaptermethod = timeadaptermethod
    def clean_up(self, solverdriver, meshcontext, report_success = 1):
        solverdriver.update_solver_state(meshcontext, report_success)
        solverdriver.clear_memory() ## deletes temporary stiffness matrix and associated objects
        
    def apply(self, solverdriver, meshcontext, verbosity = 1, report_success = 1):
        while self.timeadaptermethod.iterate(meshcontext, self.steppermethod): 
            ## iterate while the looping condition is not met
            self.timeadaptermethod.apply(self.steppermethod,
                                         solverdriver,
                                         meshcontext,
                                         verbosity,
                                         report_success)
            ## multiple steppers may be applied
            self.timeadaptermethod.update(meshcontext,
                                          self.steppermethod)
            ## tentative solution is accepted here, if ever: residual is calculated
            ## redraws, save to file, refine, checkpoints, all occur here
        ## cleanup of the core, and update of GUI related messages
        self.clean_up(solverdriver, meshcontext, report_success)
        self.redraw()
        ## solverdriver.update_solver_state(meshcontext, report_success)
        ## solverdriver.clear_memory() ## deletes temporary stiffness matrix and associated objects
        
    def apply_one_step(self, solverdriver, meshcontext, verbosity = 1, report_success = 1):
        self.timeadaptermethod.apply(self.steppermethod,
                                     solverdriver,
                                     meshcontext,
                                     verbosity,
                                     report_success)


if devel.devel(): 
    registeredclass.Registration(
        'Evolver',
        TimeDriver,
        Evolver,
        ordering=200,
        params=[
        parameter.RegisteredParameter('steppermethod', steppingscheme.TimeDependentPtr,
                                      tip ='algorithm to iterate over time'),
        parameter.RegisteredParameter('timeadaptermethod', timestepadapter.TimeStepAdapterPtr,
                                      tip ='mechanism to set time step when material is evolving in time')
        
        ],
        tip='For solving microstructural fields with implicit (properties are a function of time) or explicit (active partial differential equations have first and/or second order time-derivative(s)) time dependence'
        )
    



